#include "Loader.h"

#include <fstream>
#include <string>

namespace Loader {
	RayTracer::Image process(const char *filename, int width, int height, int oversample, int debugx, int debugy) {
		std::ifstream file(filename);
		std::string line;

		int imageSize[2] = {width, height};
		Vertex<3> cameraPosition = {0.0, 0.0, 0.0};
		Vertex<3> lookAt = {0.0, 0.0, -1.0};
		Vertex<3> up = {0.0, 1.0, 0.0};
		Vertex<3> globalAmbient = {0.0, 0.0, 0.0};
		Vertex<3> backgroundColor = {0.0, 0.0, 0.0};
		std::vector<RayTracer::Object> objects;
		std::vector<RayTracer::Light> lights;
		double defaultRefractiveIndex = 1.0;
		int maxRecursion = 4;
		double fovy = 60.0;
		double aspect = double(width) / double(height);

		RayTracer::Object object;
		Vertex<3> zero = {0.0, 0.0, 0.0};
		Vertex<3> one = {1.0, 1.0, 1.0};
		object.type = RayTracer::Object_Sphere;
		object.ambient = zero;
		object.diffuse = one;
		object.specular = one;
		object.reflective = one;
		object.transmittance = zero;
		object.specularExponent = 32.0;
		object.refractiveIndex = 1.0;
		object.texture = 0;

		if (!file) {
			std::cerr << "Error loading file: " << filename << ".\n";
		}

		while (std::getline(file, line)) {
			size_t pos = line.find('#');

			if (pos != std::string::npos) {
				line = line.substr(0, pos);
			}

			Vertex<3> A;
			Vertex<3> B;
			Vertex<3> C;
			
			if (3 == sscanf(line.c_str(), "GLOBAL_AMBIENT_COLOR %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2])) {
				globalAmbient = A;
			} else if (3 == sscanf(line.c_str(), "BACKGROUND_COLOR %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2])) {
				backgroundColor = A;
			} else if (1 == sscanf(line.c_str(), "DEFAULT_REFRACTIVE_INDEX %lf", &defaultRefractiveIndex)) {
			} else if (1 == sscanf(line.c_str(), "MAX_RECURSION_DEPTH %d", &maxRecursion)) {
			} else if (1 == sscanf(line.c_str(), "FOV %lf", &fovy)) {
			} else if (6 == sscanf(line.c_str(), "POSITIONAL_LIGHT %lf %lf %lf %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2], &B.v[0], &B.v[1], &B.v[2])) {
				RayTracer::Light light;
				light.color = A;
				light.position = B;
				lights.push_back(light);
			} else if (4 == sscanf(line.c_str(), "SPHERE %lf %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2], &object.object.sphere.radius)) {
				object.type = RayTracer::Object_Sphere;
				object.object.sphere.center = A;
				objects.push_back(object);
			} else if (9 == sscanf(line.c_str(), "TRIANGLE %lf %lf %lf %lf %lf %lf %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2], &B.v[0], &B.v[1], &B.v[2], &C.v[0], &C.v[1], &C.v[2])) {
				object.type = RayTracer::Object_Triangle;
				object.object.triangle.vertex[0] = A;
				object.object.triangle.vertex[1] = B;
				object.object.triangle.vertex[2] = C;
				object.object.triangle.normal = normalize(crossProduct(B - A, C - A));
				objects.push_back(object);
			} else if (3 == sscanf(line.c_str(), "DIFFUSE_COLOR %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2])) {
				object.diffuse = A;
			} else if (3 == sscanf(line.c_str(), "AMBIENT_COLOR %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2])) {
				object.ambient = A;
			} else if (3 == sscanf(line.c_str(), "SPECULAR_COLOR %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2])) {
				object.specular = A;
			} else if (3 == sscanf(line.c_str(), "REFLECTANCE %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2])) {
				object.reflective = A;
			} else if (3 == sscanf(line.c_str(), "TRANSMITTANCE %lf %lf %lf", &A.v[0], &A.v[1], &A.v[2])) {
				object.transmittance = A;
			} else if (1 == sscanf(line.c_str(), "REFRACTIVE_INDEX %lf", &object.refractiveIndex)) {
			} else if (1 == sscanf(line.c_str(), "SHININESS %lf", &object.specularExponent)) {
			} else if (1 == sscanf(line.c_str(), "TEXTURE %d", &object.texture)) {
			} else if (line.size()) {
				std::cout << "Unrecognized line: " << line << "\n";
			}
		}

		return RayTracer::render(cameraPosition, lookAt, up, fovy, aspect,
			imageSize, globalAmbient, backgroundColor, defaultRefractiveIndex, objects,
			lights, maxRecursion, oversample, debugx, debugy);
	}
}
